@@ -466,17 +466,28 @@ class OTSProtoBufferDecoder(object):
         if proto.HasField('analyzer_parameter') and len(proto.analyzer_parameter) > 0:
             analyzer_parameter = self._parse_analyzer_parameter(proto.analyzer, proto.analyzer_parameter)
 
+        vector_options = None
+        if proto.HasField("vector_options"):
+            vector_options = self._parse_vector_options(proto.vector_options)
+
         field_schema = FieldSchema(
             proto.field_name, self._parse_field_type(proto.field_type),
             index=proto.index, store=proto.store, is_array=proto.is_array,
             enable_sort_and_agg=proto.enable_sort_and_agg,
             analyzer=proto.analyzer, sub_field_schemas=sub_field_schemas,
             analyzer_parameter=analyzer_parameter, date_formats=date_formats,
-            is_virtual_field = is_virtual_field, source_fields = source_fields
+            is_virtual_field=is_virtual_field, source_fields=source_fields, vector_options=vector_options
         )
 
         return field_schema
 
+    def _parse_vector_options(self, proto_vector_options):
+        if proto_vector_options is None:
+            return None
+
+        return VectorOptions(data_type=proto_vector_options.data_type, metric_type=proto_vector_options.metric_type,
+                             dimension=proto_vector_options.dimension)
+
     def _parse_analyzer_parameter(self, analyzer, analyzer_parameter):
         try:
             if analyzer == AnalyzerType.SINGLEWORD:
@@ -494,7 +505,7 @@ class OTSProtoBufferDecoder(object):
             else:
                 return None
         except Exception as e:
-            error_message = 'parse analyzer_parameter failed, please contact tablestore, exception:%s, request_id: %s.' % (str(e), request_id)
+            error_message = 'parse analyzer_parameter failed, please contact tablestore, exception:%s.' % (str(e))
             self.logger.error(error_message)
             raise e
 
@@ -644,11 +655,47 @@ class OTSProtoBufferDecoder(object):
             proto_result.ParseFromString(proto.group_bys)
             self._decode_group_by_results(proto_result, group_by_results)
 
+        search_hits = []
+        if proto.search_hits is not None and len(proto.search_hits) > 0:
+            if len(proto.search_hits) == len(rows):
+                self._decode_search_hits(proto.search_hits, rows, search_hits, request_id)
+            else:
+                raise OTSServiceError("Tablestore search response occur error, row count[%d] != search_hit count[%d], request_id:%s"
+                              % (len(rows), len(proto.search_hits), request_id))
+
         search_response = SearchResponse(rows, agg_results, group_by_results, 
-                                         proto.next_token, is_all_succeed, total_count)
+                                         proto.next_token, is_all_succeed, total_count, search_hits)
         search_response.set_request_id(request_id)
         return (search_response), proto
 
+    def _decode_search_hits(self, proto_results, rows, search_hits, request_id):
+        row_index = 0
+        for hit in proto_results:
+            score = hit.score
+            nested_doc_offset = hit.nested_doc_offset
+
+            highlight_result = None
+            if hit.highlight_result is not None:
+                highlight_fields = []
+                for field in hit.highlight_result.highlight_fields:
+                    highlight_field = HighlightField(field.field_name, field.field_fragments)
+                    highlight_fields.append(highlight_field)
+                highlight_result = HighlightResult(highlight_fields)
+
+            search_inner_hits = []
+            for innerHit in hit.search_inner_hits:
+                path = innerHit.path
+                search_hits2 = []
+                self._decode_search_hits(innerHit.search_hits, [], search_hits2, request_id)
+                search_inner_hits.append(SearchInnerHit(path, search_hits2))
+
+            row = rows[row_index] if len(rows) != 0 else None
+            row_index += 1
+
+            search_hit = SearchHit(row, score, highlight_result, search_inner_hits, nested_doc_offset)
+            search_hits.append(search_hit)
+
+
     def _decode_agg_results(self, proto_result, agg_results):
         if proto_result is not None:
             for agg_result in proto_result.agg_results:
@@ -719,7 +766,7 @@ class OTSProtoBufferDecoder(object):
                 sub_agg_results = []
                 self._decode_group_by_results(item.sub_group_bys_result, sub_group_by_results)
                 self._decode_agg_results(item.sub_aggs_result, sub_agg_results)
-                
+
                 result_item = GroupByFieldResultItem(item.key, item.row_count,
                                                      sub_agg_results, sub_group_by_results)
                 result_items.append(result_item)
