@@ -12,6 +12,8 @@ import tablestore.protobuf.table_store_pb2 as pb2
 import tablestore.protobuf.table_store_filter_pb2 as filter_pb2
 import tablestore.protobuf.search_pb2 as search_pb2
 
+INT8_MAX = 127
+INT8_MIN = -128
 INT32_MAX = 2147483647
 INT32_MIN = -2147483648
 
@@ -116,6 +118,34 @@ class OTSProtoBufferEncoder(object):
                 % int32.__class__.__name__
             )
 
+    def _make_repeated_int8(self, int8_query_vector):
+        if int8_query_vector is None:
+            return None
+
+        if not isinstance(int8_query_vector, list):
+            raise OTSClientError(
+                "expect list for int8_query_vector, not %s"
+                % int8_query_vector.__class__.__name__
+            )
+
+        byte_array = bytearray(0)
+        for value in int8_query_vector:
+            if not isinstance(value, int) or (value > INT8_MAX or value < INT8_MIN):
+                raise OTSClientError(
+                    "expect int8 for int8_query_vector element, not %s"
+                    % value.__class__.__name__
+                )
+            else:
+                if value < 0:
+                    byte_array.append(value + 256)
+                else:
+                    byte_array.append(value)
+
+        if len(byte_array) != 0:
+            return bytes(byte_array)
+
+        return None
+
     def _make_repeated_column_names(self, proto, columns_to_get):
         if columns_to_get is None:
             # if no column name is given, get all primary_key_columns and attribute_columns.
@@ -377,6 +407,9 @@ class OTSProtoBufferEncoder(object):
         if field_schema.store is not None:
             proto.store = field_schema.store
 
+        if field_schema.enable_highlighting is not None:
+            proto.enable_highlighting = field_schema.enable_highlighting
+
         if field_schema.is_array is not None:
             proto.is_array = field_schema.is_array
 
@@ -402,6 +435,19 @@ class OTSProtoBufferEncoder(object):
             for source_field in field_schema.source_fields:
                 proto.source_field_names.append(source_field)
 
+        if field_schema.vector_options is not None:
+            self._make_vector_options(proto.vector_options, field_schema.vector_options)
+
+    def _make_vector_options(self, proto_vector_options, vector_options):
+        if vector_options.data_type is not None:
+            proto_vector_options.data_type = self._get_enum(vector_options.data_type)
+
+        if vector_options.metric_type is not None:
+            proto_vector_options.metric_type = self._get_enum(vector_options.metric_type)
+
+        if vector_options.dimension is not None:
+            proto_vector_options.dimension = vector_options.dimension
+
     def _make_analyzer_parameter(self, analyzer, analyzer_parameter):
         if analyzer == AnalyzerType.SINGLEWORD and isinstance(analyzer_parameter, SingleWordAnalyzerParameter):
             proto_analyzer_param = search_pb2.SingleWordAnalyzerParameter()
@@ -464,6 +510,8 @@ class OTSProtoBufferEncoder(object):
                 self._make_nested_filter(proto.geo_distance_sort.nested_filter, sorter.nested_filter)
         elif isinstance(sorter, ScoreSort):
             proto.score_sort.order = self._get_enum(sorter.sort_order)
+        elif isinstance(sorter, DocSort):
+            proto.doc_sort.order = self._get_enum(sorter.sort_order)
         else:
             raise OTSClientError(
                 "Only PrimaryKeySort and FieldSort are allowed, not %s."
@@ -843,7 +891,7 @@ class OTSProtoBufferEncoder(object):
 
         return proto
 
-    def _encode_delete_row(self, table_name, row, condition, return_type, transaction_id):
+    def _encode_delete_row(self, table_name, primary_key, condition, return_type, transaction_id):
         proto = pb2.DeleteRowRequest()
         proto.table_name = self._get_unicode(table_name)
         if condition is None:
@@ -853,7 +901,7 @@ class OTSProtoBufferEncoder(object):
         if return_type == ReturnType.RT_PK:
             proto.return_content.return_type = pb2.RT_PK
 
-        proto.primary_key = bytes(PlainBufferBuilder.serialize_for_delete_row(row.primary_key))
+        proto.primary_key = bytes(PlainBufferBuilder.serialize_for_delete_row(primary_key))
         if transaction_id is not None:
             proto.transaction_id = transaction_id
 
@@ -900,7 +948,7 @@ class OTSProtoBufferEncoder(object):
         if start_column is not None:
             proto.start_column = start_column
         if end_column is not None:
-            proto.end_colun = end_column
+            proto.end_column = end_column
         if token is not None:
             proto.token = token
         if transaction_id is not None:
@@ -976,7 +1024,7 @@ class OTSProtoBufferEncoder(object):
     def _encode_compute_splits(self, table_name, index_name):
         if table_name is None:
             raise OTSClientError("table_name must not be None")
-        
+
         proto = search_pb2.ComputeSplitsRequest()
         proto.table_name = table_name
 
@@ -1007,7 +1055,7 @@ class OTSProtoBufferEncoder(object):
         if session_id is not None:
             proto.session_id = bytes(session_id.encode('utf-8'))
 
-        return proto    
+        return proto
 
     def _encode_match_query(self, query):
         proto = search_pb2.MatchQuery()
@@ -1085,8 +1133,25 @@ class OTSProtoBufferEncoder(object):
         self._make_query(proto.query, query.query)
         if query.score_mode is not None:
             proto.score_mode = self._get_enum(query.score_mode)
+
+        if query.inner_hits is not None:
+            self._make_inner_hits(proto.inner_hits, query.inner_hits)
+
         return proto.SerializeToString()
 
+    def _make_inner_hits(self, proto, inner_hits):
+        if inner_hits.sort is not None:
+            self._make_index_sort(proto.sort, inner_hits.sort)
+
+        if inner_hits.offset is not None:
+            proto.offset = inner_hits.offset
+
+        if inner_hits.limit is not None:
+            proto.limit = inner_hits.limit
+
+        if inner_hits.highlight is not None:
+            self._make_highlight(proto.highlight, inner_hits.highlight)
+
     def _encode_wildcard_query(self, query):
         proto = search_pb2.WildcardQuery()
         proto.field_name = self._get_unicode(query.field_name)
@@ -1138,6 +1203,19 @@ class OTSProtoBufferEncoder(object):
         proto.field_name = self._get_unicode(query.field_name)
         return proto.SerializeToString()
 
+    def _encode_knn_vector_query(self, query):
+        proto = search_pb2.KnnVectorQuery()
+        proto.field_name = self._get_unicode(query.field_name)
+        proto.top_k = self._get_int32(query.top_k)
+
+        if query.float32_query_vector is not None:
+            proto.float32_query_vector.extend(query.float32_query_vector)
+
+        if query.filter is not None:
+            self._make_query(proto.filter, query.filter)
+
+        return proto.SerializeToString()
+
     def _make_query(self, proto, query):
         if isinstance(query, MatchQuery):
             proto.type = search_pb2.MATCH_QUERY
@@ -1184,6 +1262,9 @@ class OTSProtoBufferEncoder(object):
         elif isinstance(query, ExistsQuery):
             proto.type = search_pb2.EXISTS_QUERY
             proto.query = self._encode_exists_query(query)
+        elif isinstance(query, KnnVectorQuery):
+            proto.type = search_pb2.KNN_VECTOR_QUERY
+            proto.query = self._encode_knn_vector_query(query)
         else:
             raise OTSClientError(
                 "Invalid query type: %s"
@@ -1221,6 +1302,9 @@ class OTSProtoBufferEncoder(object):
         if search_query.group_bys is not None:
             self._make_group_bys(proto.group_bys, search_query.group_bys)
 
+        if search_query.highlight is not None:
+            self._make_highlight(proto.highlight, search_query.highlight)
+
         return proto.SerializeToString()
 
     def _make_aggs(self, proto, aggs):
@@ -1228,13 +1312,13 @@ class OTSProtoBufferEncoder(object):
             for agg in aggs:
                 if type(agg) not in [Max, Min, Avg, Sum, Count, DistinctCount, TopRows, Percentiles]:
                     raise OTSClientError('agg must be one of [Max, Min, Avg, Sum, Count, DistinctCount, TopRows, Percentiles]')
-                        
+
                 aggregation = proto.aggs.add()
                 aggregation.name = agg.name
                 aggregation.type = agg.type
                 aggregation.body = self._agg_to_pb_str(agg)
         else:
-            raise OTSClientError('searh_query.aggs type must be list')
+            raise OTSClientError('search_query.aggs type must be list')
 
     def _make_group_bys(self, proto, group_bys):
         if isinstance(group_bys, list):
@@ -1246,7 +1330,37 @@ class OTSProtoBufferEncoder(object):
                 group_by_proto.type = group_by.type
                 group_by_proto.body = self._group_by_to_pb_str(group_by)
         else:
-            raise OTSClientError('searh_query.group_bys type must be list')
+            raise OTSClientError('search_query.group_bys type must be list')
+
+    def _make_highlight(self, proto, highlight):
+        if not isinstance(highlight, Highlight):
+            raise OTSClientError('search_query.highlight type must be Highlight')
+
+        for highlight_parameter in highlight.highlight_parameters:
+            if not isinstance(highlight_parameter, HighlightParameter):
+                raise OTSClientError('search_query.highlight_parameter type must be HighlightParameter')
+
+            proto_highlight_parameter = proto.highlight_parameters.add()
+            proto_highlight_parameter.field_name = highlight_parameter.field_name
+
+            if highlight_parameter.number_of_fragments is not None:
+                proto_highlight_parameter.number_of_fragments = highlight_parameter.number_of_fragments
+
+            if highlight_parameter.fragment_size is not None:
+                proto_highlight_parameter.fragment_size = highlight_parameter.fragment_size
+
+            if highlight_parameter.pre_tag is not None:
+                proto_highlight_parameter.pre_tag = highlight_parameter.pre_tag
+
+            if highlight_parameter.post_tag is not None:
+                proto_highlight_parameter.post_tag = highlight_parameter.post_tag
+
+            if highlight_parameter.fragments_order is not None:
+                proto_highlight_parameter.fragments_order = self._get_enum(highlight_parameter.fragments_order)
+
+        if highlight.highlight_encoder is not None:
+            proto.highlight_encoder = self._get_enum(highlight.highlight_encoder)
+
 
     def _agg_to_pb_str(self, agg):
         if isinstance(agg, TopRows):
@@ -1288,10 +1402,11 @@ class OTSProtoBufferEncoder(object):
         return proto.SerializeToString()
 
 
-    def _encode_create_index(self, table_name, index_meta):
+    def _encode_create_index(self, table_name, index_meta, include_base_data):
         proto = pb2.CreateIndexRequest()
 
         proto.main_table_name = table_name
+        proto.include_base_data = include_base_data
         self._make_secondary_index(proto.index_meta, index_meta)
 
         return proto
@@ -1321,11 +1436,11 @@ class OTSProtoBufferEncoder(object):
         proto.transaction_id = transaction_id
 
         return proto
-    
+
     def _encode_exe_sql_query(self,query):
         proto = pb2.SQLQueryRequest()
         proto.query = query
         proto.version = 2
         return proto
 
-        
+
